@@ -4,7 +4,6 @@ This package reads and writes data formats used by the Virtual
 Observatory (VO) initiative, particularly the VOTable XML format.
 """
 
-
 from astropy import config as _config
 
 from .exceptions import (
@@ -34,6 +33,8 @@ __all__ = [
     "VOTableSpecError",
 ]
 
+VERIFY_OPTIONS = ["ignore", "warn", "exception"]  # First one is default
+
 
 class Conf(_config.ConfigNamespace):
     """
@@ -41,11 +42,10 @@ class Conf(_config.ConfigNamespace):
     """
 
     verify = _config.ConfigItem(
-        "ignore",
+        VERIFY_OPTIONS,
         "Can be 'exception' (treat fixable violations of the VOTable spec as "
         "exceptions), 'warn' (show warnings for VOTable spec violations), or "
         "'ignore' (silently ignore VOTable spec violations)",
-        aliases=["astropy.io.votable.table.pedantic", "astropy.io.votable.pedantic"],
     )
 
 
