@@ -1,17 +1,23 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-"""A set of standard astronomical equivalencies."""
+"""
+A set of standard astronomical equivalencies.
 
-from collections import UserList
+The equivalency class and all equivalency functions defined here are also
+available in (and should be used through) the `astropy.units` namespace.
+
+"""
 
 # THIRD-PARTY
 import numpy as np
 
 # LOCAL
 from astropy.constants import si as _si
+from astropy.utils import deprecated_renamed_argument
 from astropy.utils.misc import isiterable
 
 from . import astrophys, cgs, dimensionless_unscaled, misc, si
-from .core import Unit, UnitsError
+from .core import Unit
+from .errors import UnitsError
 from .function import units as function_units
 
 __all__ = [
@@ -33,11 +39,12 @@ __all__ = [
     "molar_mass_amu",
     "pixel_scale",
     "plate_scale",
+    "magnetic_flux_field",
     "Equivalency",
 ]
 
 
-class Equivalency(UserList):
+class Equivalency(list):
     """
     A container for a units equivalency.
 
@@ -50,18 +57,21 @@ class Equivalency(UserList):
     """
 
     def __init__(self, equiv_list, name="", kwargs=None):
-        self.data = equiv_list
+        super().__init__(equiv_list)
         self.name = [name]
         self.kwargs = [kwargs] if kwargs is not None else [{}]
 
     def __add__(self, other):
         if isinstance(other, Equivalency):
-            new = super().__add__(other)
-            new.name = self.name[:] + other.name
-            new.kwargs = self.kwargs[:] + other.kwargs
+            # The super() returns a list, which is really a bit weird,
+            # but that means we have to pass it back through the initializer.
+            new = self.__class__(super().__add__(other))
+            # Avoid the change to list of the name and kwargs arguments.
+            new.name = self.name + other.name
+            new.kwargs = self.kwargs + other.kwargs
             return new
         else:
-            return self.data.__add__(other)
+            return super().__add__(other)  # Let list take care.
 
     def __eq__(self, other):
         return (
@@ -152,6 +162,9 @@ def spectral():
     )
 
 
+@deprecated_renamed_argument(
+    "factor", None, since="7.0", alternative='"wav" as a "Quantity"'
+)
 def spectral_density(wav, factor=None):
     """
     Returns a list of equivalence pairs that handle spectral density
@@ -162,13 +175,15 @@ def spectral_density(wav, factor=None):
     wav : `~astropy.units.Quantity`
         `~astropy.units.Quantity` associated with values being converted
         (e.g., wavelength or frequency).
+    factor : array_like
+        If ``wav`` is a |Unit| instead of a |Quantity| then ``factor``
+        is the value ``wav`` will be multiplied with to convert it to
+        a |Quantity|.
 
-    Notes
-    -----
-    The ``factor`` argument is left for backward-compatibility with the syntax
-    ``spectral_density(unit, factor)`` but users are encouraged to use
-    ``spectral_density(factor * unit)`` instead.
+        .. deprecated:: 7.0
 
+            ``factor`` is deprecated. Pass in ``wav`` as a |Quantity|,
+            not as a |Unit|.
     """
     from .core import UnitBase
 
@@ -371,11 +386,11 @@ def doppler_radio(rest):
         return restwav * ckms / (ckms - x)
 
     def to_vel_en(x):
-        resten = rest.to_value(si.eV, equivalencies=spectral())
+        resten = rest.to_value(misc.eV, equivalencies=spectral())
         return (resten - x) / (resten) * ckms
 
     def from_vel_en(x):
-        resten = rest.to_value(si.eV, equivalencies=spectral())
+        resten = rest.to_value(misc.eV, equivalencies=spectral())
         voverc = x / ckms
         return resten * (1 - voverc)
 
@@ -383,7 +398,7 @@ def doppler_radio(rest):
         [
             (si.Hz, si.km / si.s, to_vel_freq, from_vel_freq),
             (si.AA, si.km / si.s, to_vel_wav, from_vel_wav),
-            (si.eV, si.km / si.s, to_vel_en, from_vel_en),
+            (misc.eV, si.km / si.s, to_vel_en, from_vel_en),
         ],
         "doppler_radio",
         {"rest": rest},
@@ -441,11 +456,11 @@ def doppler_optical(rest):
         return restwav * (1 + voverc)
 
     def to_vel_en(x):
-        resten = rest.to_value(si.eV, equivalencies=spectral())
+        resten = rest.to_value(misc.eV, equivalencies=spectral())
         return ckms * (resten - x) / x
 
     def from_vel_en(x):
-        resten = rest.to_value(si.eV, equivalencies=spectral())
+        resten = rest.to_value(misc.eV, equivalencies=spectral())
         voverc = x / ckms
         return resten / (1 + voverc)
 
@@ -453,7 +468,7 @@ def doppler_optical(rest):
         [
             (si.Hz, si.km / si.s, to_vel_freq, from_vel_freq),
             (si.AA, si.km / si.s, to_vel_wav, from_vel_wav),
-            (si.eV, si.km / si.s, to_vel_en, from_vel_en),
+            (misc.eV, si.km / si.s, to_vel_en, from_vel_en),
         ],
         "doppler_optical",
         {"rest": rest},
@@ -518,11 +533,11 @@ def doppler_relativistic(rest):
         return restwav * ((1 + voverc) / (1 - voverc)) ** 0.5
 
     def to_vel_en(x):
-        resten = rest.to_value(si.eV, spectral())
+        resten = rest.to_value(misc.eV, spectral())
         return (resten**2 - x**2) / (resten**2 + x**2) * ckms
 
     def from_vel_en(x):
-        resten = rest.to_value(si.eV, spectral())
+        resten = rest.to_value(misc.eV, spectral())
         voverc = x / ckms
         return resten * ((1 - voverc) / (1 + (voverc))) ** 0.5
 
@@ -530,7 +545,7 @@ def doppler_relativistic(rest):
         [
             (si.Hz, si.km / si.s, to_vel_freq, from_vel_freq),
             (si.AA, si.km / si.s, to_vel_wav, from_vel_wav),
-            (si.eV, si.km / si.s, to_vel_en, from_vel_en),
+            (misc.eV, si.km / si.s, to_vel_en, from_vel_en),
         ],
         "doppler_relativistic",
         {"rest": rest},
@@ -802,7 +817,7 @@ def temperature_energy():
     e = _si.e.value
     k_B = _si.k_B.value
     return Equivalency(
-        [(si.K, si.eV, lambda x: x / (e / k_B), lambda x: x * (e / k_B))],
+        [(si.K, misc.eV, lambda x: x / (e / k_B), lambda x: x * (e / k_B))],
         "temperature_energy",
     )
 
@@ -867,3 +882,47 @@ def plate_scale(platescale):
         "plate_scale",
         {"platescale": platescale},
     )
+
+
+def magnetic_flux_field(mu_r=1):
+    r"""
+    Convert magnetic field between magnetic field strength :math:`(\mathbf{H})` and
+    magnetic flux density :math:`(\mathbf{B})` using the relationship:
+
+    .. math::
+
+        \mathbf{B} = \mu_r \mu_0 \mathbf{H}
+
+    where:
+        - :math:`\mu_0` is the vacuum permeability, a physical constant.
+        - :math:`\mu_r` is the relative permeability of the medium, a dimensionless
+          quantity.
+
+    The default setting (:math:`\mu_r=1`) represents conditions in a vacuum.
+
+    Parameters
+    ----------
+    mu_r : float, optional
+        The relative magnetic permeability of the medium. This is a dimensionless quantity
+        and has a default value of :math:`\mu_r=1` which corresponds to free space (vacuum).
+
+    Examples
+    --------
+    >>> import astropy.units as u
+    >>> H = 1 * u.Oe
+    >>> H.to(u.G, equivalencies=u.magnetic_flux_field())  # doctest: +FLOAT_CMP
+    <Quantity 1. G>
+    >>> H.to(u.G, equivalencies=u.magnetic_flux_field(mu_r=0.8))  # doctest: +FLOAT_CMP
+    <Quantity 0.8 G>
+    >>> B = 1 * u.T
+    >>> B.to(u.A / u.m, equivalencies=u.magnetic_flux_field())  # doctest: +FLOAT_CMP
+    <Quantity 795774.71502628 A / m>
+    >>> B.to(u.A / u.m, equivalencies=u.magnetic_flux_field(mu_r=0.8))  # doctest: +FLOAT_CMP
+    <Quantity 994718.39378285 A / m>
+
+    """
+    mu0 = _si.mu0.value
+    return Equivalency(
+        [(si.T, si.A / si.m, lambda x: x / (mu_r * mu0), lambda x: x * mu_r * mu0)],
+        "magnetic_flux_field",
+    )
