@@ -5,6 +5,7 @@ This module defines structured units and quantities.
 
 # Standard library
 import operator
+from functools import cached_property
 
 import numpy as np
 
@@ -121,7 +122,7 @@ class StructuredUnit:
     Structured units share most methods with regular units::
 
         >>> su.physical_type
-        ((PhysicalType('length'), PhysicalType({'speed', 'velocity'})), PhysicalType('time'))
+        astropy.units.structured.Structure((astropy.units.structured.Structure((PhysicalType('length'), PhysicalType({'speed', 'velocity'})), dtype=[('f0', 'O'), ('f1', 'O')]), PhysicalType('time')), dtype=[('f0', 'O'), ('f1', 'O')])
         >>> su.si
         Unit("((1.49598e+11 m, 1.73146e+06 m / s), 3.15576e+07 s)")
 
@@ -297,9 +298,10 @@ class StructuredUnit:
         return self._recursively_apply(operator.attrgetter("cgs"))
 
     # Needed to pass through Unit initializer, so might as well use it.
-    def _get_physical_type_id(self):
+    @cached_property
+    def _physical_type_id(self):
         return self._recursively_apply(
-            operator.methodcaller("_get_physical_type_id"), cls=Structure
+            operator.attrgetter("_physical_type_id"), cls=Structure
         )
 
     @property
@@ -424,8 +426,8 @@ class StructuredUnit:
 
         Parameters
         ----------
-        format : `astropy.units.format.Base` instance or str
-            The name of a format or a formatter object.  If not
+        format : `astropy.units.format.Base` subclass or str
+            The name of a format or a formatter class.  If not
             provided, defaults to the generic format.
 
         Notes
